; PODPROGRAMY OBSUGI REGULATORA SIECIOWEGO
; Rafa Baranowski (C) 2005
;
; Zajmowane zasoby mikrokontrolera:
;	- licznik 1
;	- modu przerwa zewntrznych PCI
;	- 1 bajt w pamici SRAM
;
; Zaoenia:
;	- czstotliwo systemowa wynosi 1 MHz (w przypadku czstotliwoci
;	  bdcej cakowit wielokrotnoci 2 MHz wystarczy zmieni
;	  wspczynnik podziau czstotliwoci taktujcej licznika 1).
;
; Etykiety procedur:
; 	Ini_regulatora	- inicjalizacja regulatora
; 	Szczyt_zasilania	- obsuga zdarzenia szczytu zasilania (PCI int)
;	Zero_zasilania	- obsuga zdarzenia zera zasilania (TOV1 int)
;
; Wymagane stae:
;	K_REGUL		- rejestr kierunku portu regulatora
;	O_REGUL		- rejestr wyjciowy portu regulatora
;	I_REGUL		- rejestr wejciowy portu regulatora
;	REG_PWM2		- nr linii dla sygnau "PWM2" (TOV1)
;	REG_ZCD		- nr linii dla sygnau "ZCD" (PCI1)
;
; Wymagane zmienne:
;	Jasnosc - okrela poziom mocy dostarczanej do obcienia (SRAM)
;	          (moe przyjmowa warto 0...155)
;

;********************************************************
;		PROCEDURA INICJALIZACJI REGULATORA
;
; Wykorzystuje rejestry:
;	R16
;
Ini_regulatora:
	cli				; zablokuj obsug przerwa

	clr	R16
	sts	Jasnosc, R16	; zeruj warto zmiennej "Jasnosc"

	cbi	O_REGUL, REG_PWM2	; wyzeruj stan linii wyzwalajcej regulator 
	sbi	K_REGUL, REG_PWM2	; i przestaw j w tryb wyjciowy

	cbi	K_REGUL, REG_ZCD	; ustal tryb wejciowy linii detektora

	ldi	R16, 1<<PCIF
	out	GIFR, R16		; anuluj ewentualne dania przerwania PCI

	in	R16, GIMSK
	sbr	R16, 1<<PCIE0
	out	GIMSK, R16		; zezwl na obsug przerwa PCI

	clr	R16
	out	PLLCSR, R16		; zablokuj ptl fazow licznika 1

	ldi	R16, (1<<CTC1)|(1<<CS12)|(1<<CS11)|(1<<CS10)
	out	TCCR1B, R16		; uruchom licznik 1 w trybie CTC z taktowaniem
					;   zegarem systemowym przy podziale przez 64
					;   (f = 1 MHz / 64 = 15,625 kHz)
					;   licznik zlicza od wartoci 0 do OCR1C

	ldi	R16, (1<<COM1B1)|(1<<COM1B0)
	out	TCCR1A, R16		; wcz obsug linii wyzwalajcej regulator
					;   (REG_PWM2=OC1B) - bdzie ona ustawiana w
					;   stan wysoki w momencie wystpienia
					;   zdarzenia zgodnego porwnania OC1B

	ldi	R16, 254
	out	OCR1C, R16		; nastaw warto maksymaln licznika na 254
	ldi	R16, 255
	out	OCR1B, R16		; nastaw warto porwnywan, determinujc
					;   moment wyzwalania regulatora tak, aby
					;   pozostawa on na razie wyczony (=255)
	ldi	R16, (1<<TOV1)
	out	TIFR, R16		; anuluj ewentualne danie przerwania TOV1

	in	R16, TIMSK
	ori	R16, (1<<TOIE1)
	andi	R16, ~((1<<OCIE1A)|(1<<OCIE1B))
	out	TIMSK, R16		; zezwl tylko na przerwania TOV1

	sei				; odblokuj obsug przerwa

	ret


;********************************************************
;	PRZERWANIOWA OBSUGA ZDARZENIA SZCZYTU ZASILANIA (PCI int)
;
Szczyt_zasilania:
	sbic	I_REGUL, REG_ZCD	; wykonuj ponisz procedur tylko pod warunkiem,
	rjmp	Szczyt_zas_kon	;   e linia ZCD znajduje si w stanie niskim

	push	R16			; umieszczanie na stosie zawartoci rej. R16,
	in	R16, SREG
	push	R16			;   oraz rejestru SREG

	in	R16, TCNT1		; odczytaj aktualn zawarto licznika 1
	subi	R16, -int(3 * 1000/64)
	out	OCR1C, R16		; i zaaduj rejestr OCR1C tak wartoci,
					;   aby zerowanie licznika nastpio za 3 ms

	pop	R16			; ciganie ze stosu zawartoci rej. SREG,
	out	SREG, R16
	pop	R16			;   oraz rejestru R16
Szczyt_zas_kon:
	reti


;********************************************************
;	PRZERWANIOWA OBSUGA ZDARZENIA ZERA ZASILANIA (TOV1 int)
;
; Parametry:
;	Jasnosc - okrela poziom mocy dostarczanej do obcienia (SRAM)
;	          (moe przyjmowa warto 0...155)
;
Zero_zasilania:
	push	R16		; umieszczanie na stosie zawartoci rej. R16,
	push	R17		;   rejestru R17,
	in	R16, SREG
	push	R16		;   oraz rejestru SREG

	ldi	R16, 156
	lds	R17, Jasnosc; odczytaj warto zmiennej "Jasnosc"
	sub	R16, R17
	out	OCR1B, R16	; i na jej podstawie ustal warto rejestru OCR1B,
				;   okrelajcego opnienie, z jakim wyzwolony
				;   zostanie regulator

	ldi	R16, int(10 * 1000/64)
	out	OCR1C, R16	; ustal pojemno licznika 1 tak, aby kolejne
				;   przepenienie w razie braku przerwania INT0
				;   wystpio za okoo 10 ms

	ldi	R16, (1<<COM1B1)|(1<<FOC1B)
	out	TCCR1A, R16	; zeruj stan linii wyzwalajcej regulator (REG_PWM2/OC1B)
	ldi	R16, (1<<COM1B1)|(1<<COM1B0)
	out	TCCR1A, R16	; linia ta zostanie ponownie ustawiona przy
				;   zdarzeniu zgodnego porwnania OC1B

	pop	R16		; ciganie ze stosu zawartoci rej. SREG,
	out	SREG, R16
	pop	R17		;   rejestru R17,
	pop	R16		;   oraz rejestru R16
	reti
