; Tester kart telefonicznych
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU	SYS_FREQ	= 1		; czstotliwo pracy w MHz
					; stae interfejsu karty chipowej
.EQU	K_KC		= DDRA	;   rejestr kierunku portu kart 
.EQU	O_KC		= PORTA	;   rejestr wyjciowy portu kart
.EQU	I_KC		= PINA	;   rejestr wejciowy portu kart 
.EQU	KC_RS		= 0		;   nr linii dla sygnau Reset
.EQU	KC_CK		= 1		;   nr linii dla sygnau Clock
.EQU	KC_IO		= 2		;   nr linii dla sygnau Input/Output
					; stae klawiatury
.EQU	K_KLAW	= DDRA	;   rejestr kierunku portu klawiatury 
.EQU	O_KLAW	= PORTA	;   rejestr wyjciowy portu klawiatury
.EQU	I_KLAW	= PINA	;   rejestr wejciowy portu klawiatury
.EQU	KL_A		= 6		;   nr linii dla klawisza A
.EQU	KL_B		= 7		;   nr linii dla klawisza B
					; stae wywietlacza LCD
.EQU	K_LCD		= DDRB	;   rejestr kierunku portu wywietlacza
.EQU	O_LCD		= PORTB	;   rejestr wyjciowy portu wywietlacza
.EQU	LCD_DB4	= 0		;   nr linii dla sygnau DB4
.EQU	LCD_DB5	= 1		;   nr linii dla sygnau DB5
.EQU	LCD_DB6	= 2		;   nr linii dla sygnau DB6
.EQU	LCD_DB7	= 3		;   nr linii dla sygnau DB7
.EQU	LCD_RS	= 4		;   nr linii dla sygnau RS
.EQU	LCD_EN	= 5		;   nr linii dla sygnau EN

.DSEG					; SEGMENT DANYCH
.ORG 0x0060				; pocztek pamici SRAM
KC_Data:	.BYTE	16		; zmienna tablicowa zawartoci karty

.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			; skok do procedury inicjalizujcej

					; deklaracje staych w pamici programu
T_powitanie:	.db	"Tester kart tel.", ZNAK_KONCA
T_legenda1:		.db	"Kod [hex] - kl.A", ZNAK_KONCA
T_legenda2:		.db	"Jednostki - kl.B", ZNAK_KONCA
T_jednostki:	.db	"Jednostek:      ", ZNAK_KONCA 


Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	rcall	Ini_KC		; inicjalizacja interfejsu kart

	sbi	O_KLAW, KL_A
	sbi	O_KLAW, KL_B	; wcz podciganie na liniach klawiatury

	rcall	Ini_LCD		; inicjalizacja wywietlacza LCD
	ldi	R16, (1<<LCD_STER)|(1<<LCD_SW_ON)|(1<<LCD_SK_OFF)|(1<<LCD_SM_OFF)
	rcall	Instrukcja_LCD	; wyczanie kursora LCD

	ldi	ZH, high(T_powitanie<<1)
	ldi	ZL, low(T_powitanie<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie powitania

	ldi	R17, 100
	ldi	R16, 10
	rcall	Czekaj_ms		; opnienie o 1 s

Petla:				; ptla gwna programu
	rcall	Czysc_LCD		; czyszczenie wywietlacza
	ldi	ZH, high(T_legenda1<<1)
	ldi	ZL, low(T_legenda1<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie grnego wiersza legendy
	ldi	R16, 0x40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	; przejcie do wiersza dolnego
	ldi	ZH, high(T_legenda2<<1)
	ldi	ZL, low(T_legenda2<<1)
	rcall	Pokaz_ciag_LCD	; wywietlanie dolnego wiersza legendy

Ptl_spr_kl_A:
	sbic	I_KLAW, KL_A	; jeli nacinito klawisz A, to...
	rjmp	Ptl_spr_kl_B

	ldi	R16, 0x00|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	;   przesu kursor LCD do wiersza grnego

	ldi	R16, 16		;   okrel liczb danych odczytywanych z karty
	ldi	XH, high(KC_Data)
	ldi	XL, low(KC_Data)	;   zaaduj wskanik zmiennej KC_Data
	rcall	Czytaj_KC		;   odczytaj zawarto karty i zapisz j
					;     w zmiennej KC_Data

	ldi	XH, high(KC_Data)
	ldi	XL, low(KC_Data)	;   zaaduj wskanik zmiennej KC_Data
	ldi	R19, 16		;   zaaduj licznik ptli wywietlajcej
Ptl_wysw_hex_1:
	ld	R16, X+		;     odczytaj kolejny bajt zmiennej KC_Data
	rcall	Bin2hex		;     dokonaj konwersji na kod ASCII HEX
	mov	R16, R18
	rcall	Zapisz_LCD		;     wywietl jej starsz tetrad (hex)
	mov	R16, R17
	rcall	Zapisz_LCD		;     wywietl jej modsz tetrad (hex)

	cpi	R19, 9		;     jeli przekroczono dugo linii LCD,
	brne	Ptl_wysw_hex_2
	ldi	R16, 0x40|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	;       to przejd do wiersza dolnego
Ptl_wysw_hex_2:
	dec	R19
	brne	Ptl_wysw_hex_1	;   zaptlenie iteracyjne (x16)

Ptl_pusc_kl_A:
	sbis	I_KLAW, KL_A	;   oczekuj na puszczenie klawisza A
	rjmp	Ptl_pusc_kl_A
	rjmp	Petla			;   zakocz wywietlanie zawartoci karty


Ptl_spr_kl_B:
	sbic	I_KLAW, KL_B	; jeli nacinito klawisz B, to...
	rjmp	Ptl_spr_kl_A

	rcall	Czysc_LCD		;   czyszczenie zawartoci LCD

	ldi	ZH, high(T_jednostki<<1)
	ldi	ZL, low(T_jednostki<<1)
	rcall	Pokaz_ciag_LCD	;   wywietlanie etykiety jednostek
	ldi	R16, (0x00+11)|(1<<LCD_DDADR)
	rcall	Instrukcja_LCD	;   przesuwanie kursora w odpowiednie miejsce

	ldi	R16, 16		;   okrel liczb danych odczytywanych z karty
	ldi	XH, high(KC_Data)
	ldi	XL, low(KC_Data)	;   zaaduj wskanik zmiennej KC_Data
	rcall	Czytaj_KC		;   odczytaj zawarto karty i zapisz j
					;     w zmiennej KC_Data

	ldi	R16, 5		;   okrel ilo bajtw tworzcych liczb
					;     pozostaych jednostek na karcie
	ldi	XH, high(KC_Data+8)
	ldi	XL, low(KC_Data+8);   zaaduj wskanik X adresem pierwszego
					;     (najstarszego) bajtu, tworzcego
					;     liczb pozostaych jednostek
	rcall	AbaOct2Bin		;   konwertuj 5-bajtow liczb okrelajc
					;     liczb jednostek z formatu
					;     liczyda oktetowego na binarny
	rcall	Pokaz_slowo_LCD	;   wywietl liczb pozostaych jednostek

Ptl_pusc_kl_B:
	sbis	I_KLAW, KL_B	;   oczekuj na puszczenie klawisza B
	rjmp	Ptl_pusc_kl_B
	rjmp	Petla			;   zakocz wywietlanie liczby jednostek


;****************************************************************
;	PROCEDURA KONWERSJI WARTOCI W FORMACIE BIN -> ASCII HEX
;
; Parametry:
;	R16	- liczba w formacie BIN
;
; Wartoci wyjciowe:
;	R17	- zakodowana w ASCII posta HEX modszego bajtu
;	R18	- zakodowana w ASCII posta HEX starszego bajtu
;
Bin2hex:
	mov	R17, R16		; skopiuj liczb konwertowan do R17
	andi	R17, 0x0F		; zamaskuj jej starsz tetrad 
	cpi	R17, 10		; jeli liczba jest mniejsza ni 10, to...
	brsh	B2H_1
	ori	R17, 0b00110000	;   dokonaj konwersji na cyfr ASCII
	rjmp	B2H_2			; w przeciwnym razie...
B2H_1:subi	R17, -55		;   dokonaj konwersji na liter ASCII

B2H_2:mov	R18, R16		; skopiuj liczb konwertowan do R18
	andi	R18, 0xF0		; zamaskuj jej modsz tetrad
	swap	R18			; zamie pbajty
	cpi	R18, 10		; jeli liczba jest mniejsza ni 10, to...
	brsh	B2H_3
	ori	R18, 0b00110000	;   dokonaj konwersji na cyfr ASCII
	ret				; w przeciwnym razie...
B2H_3:subi	R18, -55		;   dokonaj konwersji na liter ASCII
	ret


;****************************************************************
; PROCEDURA KONWERSJI WARTOCI W FORMACIE LICZYDA OKTETOWEGO -> BIN
;
; Parametry:
;	R16	- liczba bajtw tworzcych kolejne pozycje 
;		    liczyda oktetowego (1...5)
;	X	- wskanik do zmiennej tablicowej liczyda
;	          (bajty w kolejnoci MSB...LSB)
;
; Wartoci wyjciowe:
;	R17	- LSB wartoci liczyda w formacie BIN
;	R18	- MSB wartoci liczyda w formacie BIN
;
AbaOct2bin:
	push	R19
	push	R20
	push	R21

	clr	R17
	clr	R18			; zeruj akumulator, tj. rejestr R18:R17,
					;   w ktrym znajdzie si warto
					;   liczyda w formacie binarnym
	clr	R19
	add	XL, R16
	adc	XH, R19		; przesu wskanik X na najmodsz
					;   pozycj liczyda

	ldi	R20, low(1)		; zaaduj par rejestrw R21:R20 wag
	ldi	R21, high(1)	;   aktualnie badanej pozycji liczyda
AO2B_1:				; ptla badajca kolejne pozycje liczyda 
	ld	R19, -X		;   odczytaj kolejn pozycj liczyda
AO2B_2:
	lsr	R19			;   sprawd wszystkie jej bity, a po
	brcc	AO2B_3		;     wykryciu kadego bitu w stanie
	add	R17, R20		;     wysokim zwiksz akumulator o warto
	adc	R18, R21		;     wagi aktualnie badanego bajtu
AO2B_3:
	tst	R19			
	brne	AO2B_2

	lsl	R20			;   zmodyfikuj zawarto rejestrw
	rol	R21			;     R21:R20 tak, by zawieray one
	lsl	R20			;     warto wagi kolejnej badanej
	rol	R21			;     pozycji liczyda
	lsl	R20			;     (poniewa liczydo jest oktetowe,
	rol	R21			;     to waga jest mnoona przez 8)

	dec	R16
	brne	AO2B_1		; zaptl dla kolejnego bajtu

	pop	R21
	pop	R20
	pop	R19
	ret


					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "chip_card.inc"	;   obsuga kart chipowych
.INCLUDE "lcd.inc"		;   obsuga wywietlacza LCD
