; Regulator obcienia sieciowego sterowany pilotem SIRC
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU OSC_FREQ	= 1000000	; czstotliwo pracy w Hz
.EQU SYS_FREQ	= OSC_FREQ/1000000; czstotliwo pracy w MHz
					; stae odbiornika podczerwieni
.EQU K_IR_IN	= DDRB	;   rejestr kierunku portu
.EQU O_IR_IN	= PORTB	;   rejestr wyjciowy portu
.EQU I_IR_IN	= PINB	;   rejestr wejciowy portu
.EQU IR_IN		= 6		;   nr linii dla sygnau odbieranego (INT0)
					; stae rozkazw pilota SIRC
.EQU VOL_PLUS	= 18		;   kod rozkazu zwikszania gonoci
.EQU VOL_MINUS	= 19		;   kod rozkazu zmniejszania gonoci
.EQU POWER_OFF	= 21		;   kod rozkazu wyczania
					; stae regulatora
.EQU K_REGUL	= DDRB	;   rejestr kierunku portu regulatora
.EQU O_REGUL	= PORTB	;   rejestr wyjciowy portu regulatora
.EQU I_REGUL	= PINB	;   rejestr wejciowy portu regulatora
.EQU REG_PWM2	= 3		;   nr linii dla sygnau "PWM2" (OC1B)
.EQU REG_ZCD	= 0		;   nr linii dla sygnau "ZCD" (PCI0)


.DSEG					; SEGMENT DANYCH
.ORG	0x0060			; pocztek pamici SRAM
IR_rozkaz:		.BYTE 1	;   zmienna rozkazu wiadomoci SIRC
IR_adres:		.BYTE 1	;   zmienna adresu wiadomoci SIRC
IR_znacz:		.BYTE 1	;   zmienna znacznikw wiadomoci SIRC
Jasnosc:		.BYTE	1	;   zmienna jasnoci wiecenia arwki


.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			; skok do procedury inicjalizujcej
.ORG	INT0addr			; wektor przerwania INT0
	rjmp	Odbior_IR		; skok do procedury obsugujcej odbiornik podczerwieni
.ORG	PCI0addr			; wektor przerwania PCI
	rjmp	Szczyt_zasilania	; skocz do procedury obsugujcej szczyt zasilania
.ORG	OVF1addr			; wektor przerwania TOV1
	rjmp	Zero_zasilania	; skocz do procedury obsugujcej zero zasilania


.ORG	0x30				; przejcie do pierwszej komrki za wektorami przerwa


Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	ser	R16
	out	PORTA, R16		; wcz podciganie na wszystkich portach ukadu
	out	PORTB, R16		;   (konieczne, aby nie wystpoway faszywe
					;   dania przerwania PCI)

	rcall	Ini_IR		; inicjalizacja odbiornika podczerwieni
	rcall	Ini_regulatora	; inicjalizacja regulatora sieciowego

	sei				; odblokuj przerwania

Petla:				; ptla gwna programu
	lds	R16, IR_znacz
	sbrs	R16, IR_NOWY
	rjmp	Petla			; oczekiwanie na odbir nowej wiadomoci SIRC

	cbr	R16, 1<<IR_NOWY
	sts	IR_znacz, R16	; zerowanie znacznika nowej wiadomoci

	lds	R17, IR_rozkaz	; odczytaj rozkaz nowo odebranej wiadomoci SIRC
	cpi	R17, VOL_PLUS	; jeli odebrano rozkaz zwikszenia gonoci, to...
	brne	Petla_minus

	lds	R16, Jasnosc	;   odczytaj zawarto zmiennej "Jasnosc"
	inc	R16			;   i inkrementuj j
	cpi	R16, 156
	brlo	Petla_plus_1	;   jeli inkrementowano powyej 155, to...
	ldi	R16, 155		;     zmniejsz "Jasnosc" do 155
Petla_plus_1:
	sts	Jasnosc, R16	;   zapamitaj zmodyfikowan "Jasnosc"
	rjmp	Petla			;   i wr do oczekiwania na now wiadomo

Petla_minus:
	cpi	R17, VOL_MINUS	; jeli odebrano rozkaz zmniejszenia gonoci, to...
	brne	Petla_off

	lds	R16, Jasnosc	;   odczytaj zawarto zmiennej "Jasnosc"
	dec	R16			;   i dekrementuj j
	cpi	R16, 156
	brlo	Petla_minus_1	;   jeli dekrementowano poniej zera, to...
	clr	R16			;     ustal "Jasnosc" na 0
Petla_minus_1:
	sts	Jasnosc, R16	;   zapamitaj zmodyfikowan "Jasnosc"
	rjmp	Petla			;   i wr do oczekiwania na now wiadomo

Petla_off:
	cpi	R17, POWER_OFF	; jeli odebrano rozkaz wyczenia, to...
	brne	Petla

	clr	R16
	sts	Jasnosc, R16	;   wyzeruj zmienn "Jasnosc"
	rjmp	Petla			;   i wr do oczekiwania na now wiadomo


					; DOCZANE PODPROGRAMY
.INCLUDE "czekaj_ms.inc"	;   oczekiwanie milisekundowe
.INCLUDE "czekaj_us.inc"	;   oczekiwanie mikrosekundowe
.INCLUDE "ir_sirc_x.inc"	;   obsuga odbiornika podczerwieni
.INCLUDE "reg_siec_pci.inc"	;   obsuga regulatora sieciowego
