; Prosty odtwarzacz melodii zapisanej w pamici EEPROM
; Rafa Baranowski (C) 2005

.INCLUDE "tn26def.inc"		; SEGMENT DEKLARACJI

.EQU OSC_FREQ	= 1000000	; czstotliwo pracy w Hz
					; stae gonika
.EQU K_GLOSNIK	= DDRB	;   rejestr kierunku portu
.EQU O_GLOSNIK	= PORTB	;   rejestr wyjciowy portu
.EQU GLOSNIK	= 1		;   nr linii dla sygnau audio (PB1/OC1A)
					; stae melodii
.EQU TEMPO		= 80		;   tempo [bpm] (min. 8)
.EQU DLUGOSC	= 46		;   sumaryczna liczba nut i pauz
.EQU PAUZA		= 0xFF	;   etykieta symboliczna kodu pauzy

.ESEG					; SEGMENT STAYCH W PAMICI EEPROM
.ORG 0				; ponisze wartoci stanowi zakodowan
					; melodi utworu "Dla Elizy" Beethovena;
					; kad nut tworzy para bajtw:
					; 1. bajt okrela nr ptonu w zakresie
					;    0...119, gdzie 60 to rodkowe C
					; 2. bajt okrela dugo nuty, wyraon
					;    przez liczb 1/64 czci caej nuty
					;    (4 oznacza szesnastk, 16 - wiernut)
	.DB	0x4C, 4,	0x4B, 4, 	0x4C, 4, 	0x4B, 4, 	0x4C, 4
	.DB	0x47, 4,	0x4A, 4, 	0x48, 4, 	0x45, 4, 	0x34, 4
	.DB	0x39, 4, 	0x3C, 4, 	0x40, 4, 	0x45, 4, 	0x47, 4
	.DB	0x34, 4,	0x38, 4, 	0x40, 4, 	0x44, 4, 	0x47, 4
	.DB	0x48, 4, 	0x34, 4, 	0x39, 4,	0x40, 4, 	0x4C, 4
	.DB	0x4B, 4, 	0x4C, 4, 	0x4B, 4, 	0x4C, 4, 	0x47, 4
	.DB	0x4A, 4, 	0x48, 4, 	0x45, 4,	0x34, 4, 	0x39, 4
	.DB	0x3C, 4,	0x40, 4, 	0x45, 5, 	0x47, 5, 	0x34, 5
	.DB	0x38, 6, 	0x3E, 6, 	0x48, 7, 	0x47, 7, 	0x45, 8
	.DB	PAUZA, 16


.DSEG					; SEGMENT DANYCH
.ORG	0x0060			; pocztek pamici SRAM
Wsk_melodii:	.BYTE 1	;   zmienna wskanika do kodu nuty w EEPROM
Dlugosc_nuty:	.BYTE 1	;   zmienna dugoci aktualnie odtwarzanej nuty
					;     (wyraona przez liczb 1/64 czci caej nuty)

.CSEG					; SEGMENT PROGRAMU
.ORG	0				; wektor zerowania
	rjmp	Reset			;   skok do procedury inicjalizujcej
.ORG	OVF0addr			; wektor przerwania TOV0
	rjmp	Odtw_melodie	;   skok do procedury odtwarzajcej melodi

Cz_nut:				; deklaracja tablicy wartoci, bdcych
	.DB 119, 113, 106, 100	;   wspczynnikami podziau czstotliwoci taktowania
	.DB 95, 89, 84, 80	;   licznika 1 dla kolejnych ptonw najwyszej
	.DB 75, 71, 67, 63	;   oktawy (dla nut o kodach 108...119)

Reset:				; podprogram inicjalizujcy
	ldi	R16, RAMEND
	out	SP, R16		; inicjalizacja wskanika stosu

	ldi	R16, (1<<CS02)|(1<<CS00)
	out	TCCR0, R16		; uruchom licznik 0 taktowany z czstotliwoci
					;   systemow podzielon w stosunku 1:1024
	ldi	R16, 1<<TOIE0
	out	TIMSK, R16		; zezwl na przerwania przepenienia TOV0

	ldi	R16, (1<<COM1A0)
	out	TCCR1A, R16		; docz wyjcie moduu porwnujcego OC1A licznika 1
					;   do linii sygnau audio; stan linii
					;   bdzie negowany przy kadym zgodnym porwnaniu
					;   OCR1A z TCNT1 (generacja sygnau prostoktnego)
	sbi	K_GLOSNIK, GLOSNIK; wcz tryb wyjciowy linii sygnau audio

	clr	R16
	sts	Wsk_melodii, R16	; ustaw wskanik na kod pierwszej nuty melodii
	ldi	R16, 1
	sts	Dlugosc_nuty, R16	; ustal czas opnienia odtwarzania na 1/64 caej nuty
					;   (uwaga: zeru odpowiada opnienie 256/64 c. nuty)

	sei				; odblokuj przerwania

Petla:				; ptla gwna programu
	rjmp	Petla			; (program pusty)


;********************************************************************************
Odtw_melodie:			; PROCEDURA ODTWARZAJCA MELODI (obsuga TOV0 int)
	push	R16
	push	R17
	push	R18
	in	R16, SREG
	push	R16

	ldi	R16, INT(256-((OSC_FREQ/1024)*15/(TEMPO*4)) + 0.5)
	out	TCNT0, R16		; tak ustaw licznik 0, aby kolejne przerwanie TOV0
					;   nastpio po upyniciu 1/64 czci caej nuty

	lds	R16, Dlugosc_nuty	; odczytaj czas pozostay do koca wygrywanej nuty
	dec	R16			; dekrementuj go o 1/64 cz caej nuty
	sts	Dlugosc_nuty, R16	; i zapisz z powrotem
	brne	Odtw_mel_koniec	; jeli czas si nie skoczy, to zakocz procedur

	lds	R17, Wsk_melodii	; odczytaj adres kolejnej nuty w pamici EEPROM
	rcall	Czytaj_EE		; odczytaj kod kolejnej nuty
	mov	R18, R16		; i zapamitaj go w zmiennej R18
	inc	R17			; zwiksz rejestr adresu EEPROM
	rcall	Czytaj_EE		; odczytaj czas trwania kolejnej nuty
	sts	Dlugosc_nuty, R16	; i zapamitaj go w odpowiedniej zmiennej

	inc	R17			; zwiksz rejestr adresu EEPROM
	cpi	R17, DLUGOSC*2	; porwnaj jego zawarto z dugoci melodii
	brlo	Odtw_mel_zap_wsk	; jeli to koniec melodii, to...
	clr	R17			;   zawr wskanik na jej pocztek
Odtw_mel_zap_wsk:
	sts	Wsk_melodii, R17	; zapisz zmodyfikowan warto wskanika

	cpi	R18, PAUZA		; sprawd, czy aktualna nuta jest pauz
	brne	Odtw_mel_licz	; jeli tak, to...

	clr	R16
	out	TCCR1B, R16		;   wycz generacj sygnau dwikowego
					;     poprzez wyczenie licznika 1
	rjmp	Odtw_mel_koniec	;   i zakocz procedur

Odtw_mel_licz:			; ptla obliczajca nr oktawy (0...9) i
					;   wzgldny numer ptonu w oktawie (0...11)
	clr	R16			; zeruj zmienn numeru oktawy
Odtw_mel_licz_1:
	subi	R18, 12		;   odejmij od kodu nuty liczb ptonw w oktawie
	brlo	Odtw_mel_licz_2	;   jeli wynik mniejszy od 0, to zakocz ptl
	inc	R16			;   w przeciwnym razie inkrementuj numer oktawy
	rjmp	Odtw_mel_licz_1	;     i zaptl
Odtw_mel_licz_2:			; jeli zakoczono obliczenia, to...
	subi	R18, -12		;   zmodyfikuj kod nuty tak, aby by on
					;   teraz numerem wzgldnym ptonu w oktawie

	ldi	R17, 10
	sub	R17, R16
	ori	R17, (1<<CTC1)	; na podstawie obliczonego numeru oktawy ustal
	out	TCCR1B, R17		; wspczynnik podziau cz. taktowania licznika 1

	ldi	ZH, high(Cz_nut<<1)
	ldi	ZL, low(Cz_nut<<1)
	clr	R16
	add	ZL, R18
	adc	ZH, R16
	lpm	R16, Z		; odczytaj z tablicy "Cz_nut" wspczynnik podziau
					;   cz. taktowania licznika 1 dla obliczonego,
					;   wzgldnego numeru ptonu w oktawie 
	out	OCR1C, R16		; i zaaduj nim rejestr, okrelajcy
					;   pojemno licznika 1
Odtw_mel_koniec:
	pop	R16
	out	SREG, R16
	pop	R18
	pop	R17
	pop	R16
	reti


					; DOCZANE PODPROGRAMY
.INCLUDE "eeprom.inc"		;   obsuga pamici EEPROM
